#include <Wire.h>
#include <sCtrl.h>

sCtrl_Class::sCtrl_Class(uint8_t addr)
{
    i2c_addr = addr;
}

void sCtrl_Class::begin()
{
    set(0x0000, 0x4000, 0, 0);
}


void sCtrl_Class::send(uint16_t dst_addr, uint8_t *pdata, uint8_t data_len)
{
    uint8_t i;

    Wire.beginTransmission(i2c_addr);
    Wire.write((uint8_t)(dst_addr));
    Wire.write((uint8_t)(dst_addr>>8));

    for (i=0; i<data_len; i++) {
        Wire.write(pdata[i]);
    }
    Wire.endTransmission();
}


uint8_t sCtrl_Class::receive()
{
    uint8_t i;

    i   = 0;
    len = 0;

    Wire.requestFrom(i2c_addr, (uint8_t)1);
    if (Wire.available()) {
        if (len = Wire.read()) {
            Wire.requestFrom(i2c_addr, len);
            while (Wire.available() && i < SCTRL_RX_BUF_SIZE) {
                buf[i] = Wire.read();
                i++;
            }

            if (i == len) {
                src_addr = (uint16_t)buf[1] << 8 | buf[0];
                pdata    = &buf[2];
                data_len = len - 2;
                return true;
            }
        }
    }

    return false;
}


uint8_t sCtrl_Class::parse()
{
    sCtrl_Msg_t *pmsg = (sCtrl_Msg_t *)buf;

    switch (pmsg->action) {
    case SCTRL_GET:
        if (data_len == 3) {
            action  = SCTRL_GET;
            reg     = pmsg->reg;
            pval    = pmsg->pval;
            val_len = 0;
            return true;
        }
        break;

    case SCTRL_SET:
        if (data_len >= 3) {
            action  = SCTRL_SET;
            reg     = pmsg->reg;
            pval    = pmsg->pval;
            val_len = data_len - 3;
            return true;
        }
        break;

    case SCTRL_REPORT:
        if (data_len >= 3) {
            action  = SCTRL_REPORT;
            reg     = pmsg->reg;
            pval    = pmsg->pval;
            val_len = data_len - 3;
            return true;
        }
        break;

    default:
        break;
    }

    return false;
}


void sCtrl_Class::get(uint16_t addr, uint16_t reg)
{
    write(SCTRL_GET, addr, reg, (uint8_t *)0, 0);
}


void sCtrl_Class::set(uint16_t addr, uint16_t reg, uint8_t *pval, uint8_t val_len)
{
    write(SCTRL_SET, addr, reg, pval, val_len);
}


void sCtrl_Class::report(uint16_t addr, uint16_t reg, uint8_t *pval, uint8_t val_len)
{
    write(SCTRL_REPORT, addr, reg, pval, val_len);
}


uint8_t sCtrl_Class::INT8U (uint8_t *ptr)
{
    return *ptr;
}


uint16_t sCtrl_Class::INT16U (uint8_t *ptr)
{
    return *(uint16_t *)ptr;
}


void sCtrl_Class::write(uint8_t action, uint16_t addr, uint16_t reg, uint8_t *pval, uint8_t val_len)
{
    uint8_t i;

    Wire.beginTransmission(i2c_addr);
    Wire.write((uint8_t)(addr));
    Wire.write((uint8_t)(addr>>8));
    Wire.write(action);
    Wire.write((uint8_t)(reg));
    Wire.write((uint8_t)(reg>>8));

    for (i=0; i<val_len; i++) {
        Wire.write(pval[i]);
    }
    Wire.endTransmission();
}

sCtrl_Class sCtrl = sCtrl_Class(0x01);
